<?php
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Class mrpu_Bulk_Usernames_Admin
 *
 * Handles the custom WP admin page for
 * bulk generation and management of random usernames.
 */
class mrpu_Bulk_Usernames_Admin {

    /**
     * Initialize hooks for adding the page
     * and loading the list table on that page.
     */
    public function init_hooks() {
        // Add page under Tools
        add_management_page(
            __('Generate Usernames', 'my-random-post-usernames'),
            __('Generate Usernames', 'my-random-post-usernames'),
            'manage_options',
            'mrpu-bulk-usernames',
            [$this, 'render_admin_page']
        );
        
        // Register AJAX for inline editing
        add_action('wp_ajax_mrpu_update_username_inline', [$this, 'ajax_update_username_inline']);
    }

    /**
     * Render the main admin page.
     */
    public function render_admin_page() {
        // Security check
        if ( ! current_user_can( 'manage_options' ) ) {
            wp_die( esc_html__( 'Unauthorized user', 'my-random-post-usernames' ) );
        }
        
        // Set active tab safely before outputting the tabs.
        $active_tab = 'bulk_processing'; // Default tab
        if ( isset( $_GET['tab'], $_GET['_wpnonce'] ) ) {
            $nonce = sanitize_text_field( wp_unslash( $_GET['_wpnonce'] ) );
            if ( wp_verify_nonce( $nonce, 'tab_switch' ) ) {
                $requested_tab = sanitize_text_field( wp_unslash( $_GET['tab'] ) );
                if ( in_array( $requested_tab, [ 'bulk_processing', 'display_settings' ], true ) ) {
                    $active_tab = $requested_tab;
                }
            }
        }
        
        require_once plugin_dir_path(__FILE__) . 'class-mrpu-posts-table.php';
        
        // Prepare and display the list table.
        $posts_table = new mrpu_Posts_Table();
        $posts_table->prepare_items();
        
        // Additional logic to get counts of posts.
        $counts = $this->get_posts_counts();
        
        ?>
        <div class="wrap">
            <div class="mrpu-donation-link" style="position: absolute; top: 20px; right: 20px; text-align: right;">
                <a href="https://www.paypal.com/donate/?hosted_button_id=7GYW2YYC6NHGJ" 
                   target="_blank"
                   style="text-decoration: none; color: #2d2d2d; display: inline-flex; align-items: center; background: #f8f9fa; padding: 8px 15px; border-radius: 4px; border: 1px solid #ddd; transition: all 0.3s ease;">
                    <span style="margin-right: 8px;">
                        Pay what you like &mdash; 
                        Made with <span style="color: #e25555; margin: 0 3px;">♥</span> 
                        by Mohamed C.
                    </span>
                    <svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" version="1.1" id="Layer_1" x="0px" y="0px" width="30px" height="33px" viewBox="0 0 45 33" enable-background="new 0 0 30 33" xml:space="preserve">
                        <path fill="#253B80" d="M7.266,29.154l0.523-3.322l-1.165-0.027H1.061L4.927,1.292C4.939,1.218,4.978,1.149,5.035,1.1  c0.057-0.049,0.13-0.076,0.206-0.076h9.38c3.114,0,5.263,0.648,6.385,1.927c0.526,0.6,0.861,1.227,1.023,1.917  c0.17,0.724,0.173,1.589,0.007,2.644l-0.012,0.077v0.676l0.526,0.298c0.443,0.235,0.795,0.504,1.065,0.812  c0.45,0.513,0.741,1.165,0.864,1.938c0.127,0.795,0.085,1.741-0.123,2.812c-0.24,1.232-0.628,2.305-1.152,3.183  c-0.482,0.809-1.096,1.48-1.825,2c-0.696,0.494-1.523,0.869-2.458,1.109c-0.906,0.236-1.939,0.355-3.072,0.355h-0.73  c-0.522,0-1.029,0.188-1.427,0.525c-0.399,0.344-0.663,0.814-0.744,1.328l-0.055,0.299l-0.924,5.855l-0.042,0.215  c-0.011,0.068-0.03,0.102-0.058,0.125c-0.025,0.021-0.061,0.035-0.096,0.035H7.266z"/>
                        <path fill="#179BD7" d="M23.048,7.667L23.048,7.667L23.048,7.667c-0.028,0.179-0.06,0.362-0.096,0.55  c-1.237,6.351-5.469,8.545-10.874,8.545H9.326c-0.661,0-1.218,0.48-1.321,1.132l0,0l0,0L6.596,26.83l-0.399,2.533  c-0.067,0.428,0.263,0.814,0.695,0.814h4.881c0.578,0,1.069-0.42,1.16-0.99l0.048-0.248l0.919-5.832l0.059-0.32  c0.09-0.572,0.582-0.992,1.16-0.992h0.73c4.729,0,8.431-1.92,9.513-7.476c0.452-2.321,0.218-4.259-0.978-5.622  C24.022,8.286,23.573,7.945,23.048,7.667z"/>
                        <path fill="#222D65" d="M21.754,7.151c-0.189-0.055-0.384-0.105-0.584-0.15c-0.201-0.044-0.407-0.083-0.619-0.117  c-0.742-0.12-1.555-0.177-2.426-0.177h-7.352c-0.181,0-0.353,0.041-0.507,0.115C9.927,6.985,9.675,7.306,9.614,7.699L8.05,17.605  l-0.045,0.289c0.103-0.652,0.66-1.132,1.321-1.132h2.752c5.405,0,9.637-2.195,10.874-8.545c0.037-0.188,0.068-0.371,0.096-0.55  c-0.313-0.166-0.652-0.308-1.017-0.429C21.941,7.208,21.848,7.179,21.754,7.151z"/>
                        <path fill="#253B80" d="M9.614,7.699c0.061-0.393,0.313-0.714,0.652-0.876c0.155-0.074,0.326-0.115,0.507-0.115h7.352  c0.871,0,1.684,0.057,2.426,0.177c0.212,0.034,0.418,0.073,0.619,0.117c0.2,0.045,0.395,0.095,0.584,0.15  c0.094,0.028,0.187,0.057,0.278,0.086c0.365,0.121,0.704,0.264,1.017,0.429c0.368-2.347-0.003-3.945-1.272-5.392  C20.378,0.682,17.853,0,14.622,0h-9.38c-0.66,0-1.223,0.48-1.325,1.133L0.01,25.898c-0.077,0.49,0.301,0.932,0.795,0.932h5.791  l1.454-9.225L9.614,7.699z"/>
                    </svg>
                </a>
            </div>
            
            <h1><?php esc_html_e('Random Post Usernames', 'my-random-post-usernames'); ?></h1>
            
            <h2 class="nav-tab-wrapper">
                <?php
                // Define the tabs.
                $tabs = [
                    'bulk_processing' => __('Bulk Processing', 'my-random-post-usernames'),
                    'display_settings' => __('Display Settings', 'my-random-post-usernames')
                ];
                
                // Output the tabs.
                foreach ( $tabs as $tab_key => $tab_name ) {
                    $class = ( $active_tab === $tab_key ) ? 'nav-tab nav-tab-active' : 'nav-tab';
                    $url = add_query_arg([
                        'page'    => 'mrpu-bulk-usernames',
                        'tab'     => $tab_key,
                        '_wpnonce'=> wp_create_nonce('tab_switch')
                    ], admin_url('tools.php'));
                    echo '<a href="' . esc_url($url) . '" class="' . esc_attr($class) . '">' . esc_html($tab_name) . '</a>';
                }
                ?>
            </h2>
        
            <div class="mrpu-admin-content">
                <?php if ($active_tab === 'bulk_processing') : ?>
                    <!-- Bulk Processing Tab Content -->
                    <div class="mrpu-bulk-processing-tab">
                        <!-- Display quick stats -->
                        <div class="mrpu-stats-container">
                            <span id="total_posts_without" style="display:none;"><?php echo intval($counts['without']); ?></span>
                            <span id="total_posts_with" style="display:none;"><?php echo intval($counts['with']); ?></span>
                            <?php
                            // translators: %d: number of posts without usernames.
                            printf(esc_html__( 'Posts without usernames: %s', 'my-random-post-usernames' ),'<strong>' . intval( $counts['without'] ) . '</strong>');
                            ?>
                            <?php
                            // translators: %d: number of posts with usernames.
                            printf(esc_html__( 'Posts with usernames: %s', 'my-random-post-usernames' ),'<strong>' . intval( $counts['with'] ) . '</strong>');
                            ?>
                        </div>
                        <!-- Bulk actions buttons -->
                        <div class="mrpu-bulk-actions">
                            <button id="mrpu-generate-missing" class="button button-primary">
                                <?php esc_html_e('Generate Usernames for New Posts', 'my-random-post-usernames'); ?>
                            </button>
                            <button id="mrpu-regenerate-existing" class="button button-secondary">
                                <?php esc_html_e('Regenerate All Usernames', 'my-random-post-usernames'); ?>
                            </button>
                        </div>
                        <!-- Optionally, you can keep the progress bar if desired -->
                        <div id="mrpu-progress-bar" style="margin-top: 20px;">
                            <div class="progress-label"><?php esc_html_e('Processing...', 'my-random-post-usernames'); ?></div>
                            <div class="progress">
                                <div class="progress-bar" role="progressbar"></div>
                            </div>
                            <div class="processed-count">
                                <?php esc_html_e('Processed:', 'my-random-post-usernames'); ?> <span>0</span>/<span class="mrpu-total"></span>
                            </div>
                        </div>
                        <!-- The WP_List_Table -->
                        <form method="get">
                            <input type="hidden" name="page" value="mrpu-bulk-usernames">
                            <?php $posts_table->display(); ?>
                        </form>
                    </div>
        
                <?php else : ?>
                    <!-- Display Settings Tab Content -->
                    <div class="mrpu-display-settings-container">
                        <div class="mrpu-settings-form-container">
                            <form method="post" action="options.php">
                                <?php
                                settings_fields('mrpu_display_settings');
                                echo '<div class="mrpu-form-content">';
                                do_settings_sections('mrpu-bulk-usernames');
                                echo '</div>';
                                submit_button();
                                ?>
                            </form>
                            <!-- New Reset Button Container -->
                            <div id="mrpu-reset-container" style="text-align: right; margin-top: 20px;">
                                <button id="mrpu-reset-usernames" class="button button-danger" style="background-color: #d63638; color: #fff;">
                                    <?php esc_html_e('Reset Usernames', 'my-random-post-usernames'); ?>
                                </button>
                            </div>
                        </div>
                        
                        <!-- Preview Container -->
                        <div id="mrpu-style-preview">
                            <h3><?php esc_html_e('Live Preview', 'my-random-post-usernames'); ?></h3>
                            <div class="mrpu-uploader-preview">
                                <span class="mrpu-posted-by-preview"><?php esc_html_e('Uploaded by', 'my-random-post-usernames'); ?></span>
                                <span class="mrpu-username-preview">example_user123</span>
                            </div>
                        </div>
                    </div>
                <?php endif; ?>
            </div>
        </div>
        <?php
    }

    /**
     * Retrieves counts of how many posts have or don’t have random usernames.
     */
    private function get_posts_counts() {
    // Define a unique cache key
    $cache_key = 'mrpu_posts_counts';
    
    // Attempt to retrieve data from the cache
    $counts = wp_cache_get($cache_key);
    
    if ($counts === false) {
        // Data not in cache, perform queries
        $with_usernames = get_posts([
            'post_type'   => 'post',
            'post_status' => 'publish',
            'meta_key'    => '_random_username',
            'fields'      => 'ids', // Only retrieve post IDs
            'numberposts' => -1     // Retrieve all matching posts
        ]);
        
        $without_usernames = get_posts([
            'post_type'   => 'post',
            'post_status' => 'publish',
            'meta_query'  => [
                [
                    'key'     => '_random_username',
                    'compare' => 'NOT EXISTS'
                ]
            ],
            'fields'      => 'ids', // Only retrieve post IDs
            'numberposts' => -1     // Retrieve all matching posts
        ]);
        
        // Count the number of posts
        $counts = [
            'with'    => count($with_usernames),
            'without' => count($without_usernames),
        ];
        
        // Store the counts in cache for 1 hour
        wp_cache_set($cache_key, $counts, '', HOUR_IN_SECONDS);
    }
    
    return $counts;
}
}