<?php
if (!defined('ABSPATH')) {
    exit;
}

if (!class_exists('WP_List_Table')) {
    require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}

class mrpu_Posts_Table extends WP_List_Table {

    public function __construct() {
        parent::__construct([
            'singular' => 'mrpu_post',
            'plural'   => 'mrpu_posts',
            'ajax'     => false,
        ]);

        add_action('admin_head', [$this, 'admin_head_styles']);
    }

    public function get_columns() {
        return [
            'cb' => '<input type="checkbox">',
            'title' => __('Post Title', 'my-random-post-usernames'),
            'username' => __('Random Username', 'my-random-post-usernames')
        ];
    }

    public function get_hidden_columns() {
        return [];
    }

    public function get_sortable_columns() {
        return [];
    }

    public function prepare_items() {
        $this->_column_headers = [
            $this->get_columns(),
            $this->get_hidden_columns(),
            $this->get_sortable_columns()
        ];
    
        $per_page = 20;
        $current_page = $this->get_pagenum();
        $offset = ($current_page - 1) * $per_page;
    
        // Get total items count using WP_Query
        $total_query = new WP_Query(array(
            'post_type'      => 'post',
            'post_status'    => 'publish',
            'posts_per_page' => -1, // Get all posts
            'fields'         => 'ids', // Only retrieve post IDs
        ));
        $total_items = $total_query->found_posts;
    
        // Get items with processing status using WP_Query
        $items_query = new WP_Query(array(
            'post_type'      => 'post',
            'post_status'    => 'publish',
            'posts_per_page' => $per_page,
            'offset'         => $offset,
            'orderby'        => 'ID',
            'order'          => 'DESC',
            'fields'         => 'ids', // Only retrieve post IDs
        ));
    
        $this->items = array();
    
        // Loop through posts and build the items array
        foreach ($items_query->posts as $post_id) {
            $post_title = get_the_title($post_id);
            $random_username = get_post_meta($post_id, '_random_username', true);
            $is_processing = (int) get_post_meta($post_id, '_mrpu_processing', true);
    
            $this->items[] = array(
                'ID'              => $post_id,
                'post_title'      => $post_title,
                'random_username' => $random_username,
                'is_processing'   => $is_processing,
            );
        }
    
        $this->set_pagination_args([
            'total_items' => $total_items,
            'per_page'    => $per_page,
            'total_pages' => ceil($total_items / $per_page),
        ]);
    }

    public function column_default($item, $column_name) {
        switch ($column_name) {
            case 'title':
                return '<a href="'.get_edit_post_link($item['ID']).'" class="post-title-link">'.esc_html($item['post_title']).'</a>';
            
            case 'username':
                if ($item['is_processing']) {
                    return '<div class="mrpu-processing">
                                <span class="spinner is-active"></span>
                                '.__('Generating username...', 'my-random-post-usernames').'
                            </div>';
                }
                
                if (empty($item['random_username'])) {
                    return sprintf(
                        '<em>%s</em>
                         <a href="#" class="mrpu-edit-username">%s</a>',
                        __('No username', 'my-random-post-usernames'),
                        __('Add', 'my-random-post-usernames')
                    );
                }
                
                return sprintf(
                    '<div class="mrpu-username-cell">
                        <span class="mrpu-username-display">%s</span>
                        <a href="#" class="mrpu-edit-username">%s</a>
                    </div>',
                    esc_html($item['random_username']),
                    __('Edit', 'my-random-post-usernames')
                );
            
            default:
                return parent::column_default($item, $column_name);
        }
    }

    public function column_cb($item) {
        return sprintf('<input type="checkbox" name="post_ids[]" value="%d">', $item['ID']);
    }

    public function display_rows() {
        foreach ($this->items as $item) {
            echo '<tr data-post_id="'.esc_attr($item['ID']).'">';
            $this->single_row_columns($item);
            echo '</tr>';
        }
    }

    public function display() {
        echo '<div class="mrpu-modern-container">';
        parent::display();
        echo '</div>';
    }
}