jQuery(document).ready(function($){
    // Initialize color pickers
    $('.mrpu-color-field').wpColorPicker({
        change: function(event, ui) {
            $(this).val(ui.color.toString()).trigger('change');
        }
    });

    // Live preview updates
    function updatePreview() {
        const $preview = $('.mrpu-uploader-preview');
        const $form = $('form[action="options.php"]');
        
        // Container styles
        const bgColor = $form.find('input[name="mrpu_display_options[background_color]"]').val();
        const borderRadius = $form.find('input[name="mrpu_display_options[border_radius]"]').val() + 'px';
        const containerFontSize = $form.find('input[name="mrpu_display_options[font_size]"]').val() + 'px';
        const containerPadding = $form.find('input[name="mrpu_display_options[container_padding]"]').val() + 'px'; // New padding
        
        // "Uploaded by" text
        const textColor = $form.find('input[name="mrpu_display_options[text_color]"]').val();
        const customText = $form.find('input[name="mrpu_display_options[custom_text]"]').val();
        
        // Username-specific styles
        const usernameColor = $form.find('input[name="mrpu_display_options[username_color]"]').val();
        const usernameFontSize = $form.find('input[name="mrpu_display_options[username_font_size]"]').val() + 'px';
        const usernameFontWeight = $form.find('select[name="mrpu_display_options[username_font_weight]"]').val();
    
        // Apply container styles
        $preview.css({
            'background-color': bgColor,
            'border-radius': borderRadius,
            'font-size': containerFontSize,
            'padding': containerPadding // Apply padding
        });
    
        // Apply "Uploaded by" text
        $preview.find('.mrpu-posted-by-preview')
               .css('color', textColor)
               .text(customText ? customText.trim() + ' ' : '');
    
        // Apply username-specific styles
        $preview.find('.mrpu-username-preview').css({
            'color': usernameColor,
            'font-size': usernameFontSize,
            'font-weight': usernameFontWeight
        });
    }

    // Update preview on any input change
    $('form[action="options.php"]').on('input change', 'input, textarea', updatePreview);
    
    // Initial update
    updatePreview();
    
    $('#generate_new_username').click(function() {
        $.post(ajaxurl, { action: 'generate_new_username' }, function(response) {
            $('#random_username').val(response);
        });
    });
    
    $('#mrpu-reset-usernames').on('click', function(e) {
        e.preventDefault();
        if (confirm(mrpu_admin_vars.resetConfirmation)) {
            $.post(mrpu_admin_vars.ajaxurl, {
                action: 'reset_usernames',
                nonce: mrpu_admin_vars.resetNonce,
            }, function(response) {
                if (response.success) {
                    alert(mrpu_admin_vars.resetSuccess);
                    location.reload();
                } else {
                    alert(mrpu_admin_vars.resetError);
                }
            });
        }
    });
});

(function($) {
    var isProcessing = false;
    var processed = 0;
    var batchSize = 20;
    var totalPosts = 0;
    var regenerate = false;

    $('#mrpu-generate-missing, #mrpu-regenerate-existing').on('click', function() {
        if (isProcessing) return;

        regenerate = ($(this).attr('id') === 'mrpu-regenerate-existing');
        // Assume you pass total counts via localized variables or elements
        totalPosts = regenerate ? parseInt($('#total_posts_with').text()) : parseInt($('#total_posts_without').text());
        
        if (totalPosts === 0) {
            alert('No posts to process.');
            return;
        }

        processed = 0;
        isProcessing = true;
        $('.progress-label').text(regenerate ? 'Regenerating existing usernames...' : 'Generating missing usernames...');
        $('#mrpu-generate-missing, #mrpu-regenerate-existing').prop('disabled', true);
        $('#mrpu-progress-bar').show();
        $('.processed-count span:first').text(0);
        $('.mrpu-total').text(totalPosts);
        $('.progress-bar').width('0%').text('0%');

        processBatch();
    });

    function processBatch() {
        $.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'process_username_batch',
                batch_size: batchSize,
                processed_so_far: processed,
                regenerate: regenerate,
                nonce: mrpu_admin_vars.nonce_bulk
            },
            success: function(response) {
                if (response.success && response.data) {
                    processed = response.data.total_processed;
                    updateProgress(processed);

                    if (response.data.logs && response.data.logs.length) {
                        response.data.logs.forEach(function(log) {
                            $('#mrpu-log-list').append('<li>' + log + '</li>');
                        });
                    }

                    if (processed < totalPosts) {
                        setTimeout(processBatch, 200);
                    } else {
                        isProcessing = false;
                        $('.progress-label').text('Complete!');
                        $('#mrpu-log-list').append('<li>All posts processed</li>');
                        setTimeout(function(){ location.reload(); }, 1200);
                    }
                } else {
                    handleError(response.data ? response.data : 'Unknown error');
                }
            },
            error: function(xhr, status, error) {
                handleError(error);
            }
        });
    }

    function updateProgress(count) {
        var percent = Math.min((count / totalPosts) * 100, 100);
        $('.progress-bar').css('width', percent + '%').text(Math.round(percent) + '%');
        $('.processed-count span:first').text(count);
    }

    function handleError(msg) {
        isProcessing = false;
        $('#mrpu-generate-missing, #mrpu-regenerate-existing').prop('disabled', false);
        $('.progress-label').html('<span style="color:red;">Error: ' + msg + '</span>');
    }
})(jQuery);

jQuery(function($) {
    $(".mrpu-edit-username").click(function(e) {
        e.preventDefault();
        const cell = $(this).closest("td");
        const postId = cell.closest("tr").data("post_id");
        const currentName = cell.find(".mrpu-username-display").text();
        
        cell.html(`
            <div class="mrpu-edit-container">
                <input type="text" class="mrpu-edit-input" value="${currentName}" placeholder="Enter username">
                <button type="button" class="mrpu-save-btn" data-post-id="${postId}">Save</button>
                <span class="spinner" style="margin-left:5px"></span>
            </div>
        `);
    });

    // Handler for clicking "Save"
    $(document).on("click", ".mrpu-save-btn", function() {
        const button = $(this);
        const postId = button.data("post-id");
        const input = button.siblings(".mrpu-edit-input");
        const spinner = button.siblings(".spinner");
        
        spinner.addClass("is-active");
        
        $.post(ajaxurl, {
            action: "mrpu_save_username",
            post_id: postId,
            username: input.val(),
            security: mrpu_admin_vars.nonce_edit
        }, function(response) {
            spinner.removeClass("is-active");
            if(response.success) {
                const displayContent = input.val() ? 
                    `<span class="mrpu-username-display">${input.val()}</span>
                     <a href="#" class="mrpu-edit-username">Edit</a>` :
                    `<em>${mrpu_vars.no_username}</em>
                     <a href="#" class="mrpu-edit-username">Add</a>`;
                button.closest("td").html(displayContent);
            } else {
                alert("Error saving username");
                button.closest("td").html(input.val() ? input.val() : "<em>No username</em>");
            }
        });
    });
});

